!------------------------------------------------------------------
! DESCRIPTION:	
!	program to compute the moveout of a multiple
!	in subsurface-offset common-image gathers and angle-domain
!	common-image gathers.
!------------------------------------------------------------------
!	Input:
!		ngth: number of gathers to process
!		nh: number of subsurface half-offsets
!		oh: minimum subsurface half-offset (negative, meters)
!		dh:subsurface half-offset sampling interval (m)
!		ng: number of subsurface half apperture angles
!		og: minimum half aperture angle (degrees)
!		dg: half aperture angle sampling interval (degrees)
!		dh:subsurface half-offset sampling interval (m)
!		v1: velocity of the water (m/s)
!		v2: velocity of the sediments (m/s)
!		zwb: depth of the water-bottom (m)
!
!	Example:
!-----------------------------------------------------------------
program image_space_moveouts

	use sep
	use image_moveouts

	implicit none

	!! program arguments
	real	:: xmzi
	real	:: xhi
	real	:: xg
	integer	:: nhi
	real	:: ohi
	real	:: dhi
	integer	:: ng
	real	:: og
	real	:: dg
	integer	:: nmd
	real	:: omd
	real	:: dmd
	integer	:: nhd
	real	:: ohd
	real	:: dhd
	real	:: v1
	real	:: v2
	real	:: zwb
	real	:: dip
	real	:: xdiff
	integer	:: type

	!! internal variables
	real							:: rho, rho2
	integer							:: ig, ihi, i 
	real,allocatable,dimension(:)	:: zih,zig,zimh,zimg
	real,allocatable,dimension(:)	:: hi,g,mzih,mzig
	integer							:: count_hi,count_g,count_mzih,count_mzig

	!! initialize SEP stuff
	call initpar()

	!! get number of gathers to process
	call from_param("xmzi" ,xmzi,   0.)
	call from_param("xhi"  ,xhi,    0.)
	call from_param("xg"   ,xg,     0.)
	call from_param("nhi"  ,nhi ,    1)
	call from_param("ohi"  ,ohi ,   0.)
	call from_param("dhi"  ,dhi ,  10.)
	call from_param("ng"   ,ng  ,    1)
	call from_param("og"   ,og  ,   0.)
	call from_param("dg"   ,dg  ,   1.)
	call from_param("nmd"  ,nmd ,    1)
	call from_param("omd"  ,omd ,   0.)
	call from_param("dmd"  ,dmd ,   1.)
	call from_param("nhd"  ,nhd ,    1)
	call from_param("ohd"  ,ohd ,   0.)
	call from_param("dhd"  ,dhd ,   1.)
	call from_param("v1"   ,v1  ,1500.)
	call from_param("v2"   ,v2  ,2500.)
	call from_param("zwb"  ,zwb ,   0.)
	call from_param("dip"  ,dip ,   0.)
	call from_param("xdiff",xdiff,  0.)
	call from_param("type",type,     1)

	! compute aux varibles
	rho=v2/v1;
	rho2=rho**2

	! allocate work space
	allocate(zih(nhi),zig(ng),zimh(nmd),zimg(nmd))
	allocate(hi(nhi),g(ng),mzih(nmd),mzig(nmd))

	! convert from degrees to radians
	og=og*PI/180.
	dg=dg*PI/180.
	xg=xg*PI/180.
	if (type>2) then
		dip=dip*PI/180.
	end if
		
	!
	! compute the moveouts	
	!
	if (type==1) then	! specularly-reflected multiple
						! from flat water bottom

		call flat_wb_cigs_moveout(nmd,omd,dmd,ng,og,dg,nhi,ohi,dhi, &
			& zwb,rho,zih,zig,zimh,zimg,hi,g,mzih,mzig)
			
		! set counters
		count_hi=nhi; count_g=ng; count_mzih=nmd; count_mzig=nmd

	else if (type==2) then	! diffracted multiple from flat WB

		call flat_diffmul_cigs_moveout(xmzi,xhi,xg,nmd,omd,dmd,&
			& nhd,ohd,dhd,xdiff,zwb,rho,v1,zih,zig,zimh,zimg,&
			& nhi,dhi,hi,ng,dg,g,mzih,mzig,count_hi,count_g,count_mzih,&
			& count_mzig)

		else if (type==3) then	! SRM from dipping water-bottom
		
		call dip_wb_cigs_moveout(xmzi,xhi,xg,nmd,omd,dmd,nhd,ohd,dhd, &
			& dip,rho,v1,zih,zig,zimh,zimg,nhi,dhi,hi,ng,dg,g,mzih,mzig,&
			& count_hi,count_g,count_mzih,count_mzig)
write(0,*)'AFTER COMPUTATUIOn'
write(0,*)'count_hi=',count_hi,'count_g=',count_g,'count_mzih=',count_mzih,'count_mzig=',count_mzig

	else if (type==4) then	! diffracted multiple from dipping WB
		
		call dip_diffmul_cigs_moveout(xmzi,xhi,xg,nmd,omd,dmd,&
			& nhd,ohd,dhd,xdiff,dip,rho,v1,zih,zig,zimh,zimg,&
			& nhi,dhi,hi,ng,dg,g,mzih,mzig,count_hi,count_g,count_mzih,&
			& count_mzig)
	else
		write(0,*)'ERROR: type has to be 1-4'
		call exit(9) 
	end if

	!! write parameters to output files
	call to_history("n1",count_hi  ,"odcig")
	call to_history("o1",0.        ,"odcig")
	call to_history("d1",1.        ,"odcig")
	call to_history("n2",1         ,"odcig")
	call to_history("o2",0.        ,"odcig")
	call to_history("d2",1.        ,"odcig")
	call to_history("esize",8      ,"odcig")
	call to_history("n1",count_g   ,"adcig")
	call to_history("o1",0.        ,"adcig")
	call to_history("d1",1.        ,"adcig")
	call to_history("n2",1         ,"adcig")
	call to_history("o2",0.        ,"adcig")
	call to_history("d2",1.        ,"adcig")
	call to_history("esize",8      ,"adcig")
	call to_history("n1",count_mzih,"image_h")
	call to_history("o1",0.        ,"image_h")
	call to_history("d1",1.        ,"image_h")
	call to_history("n2",1         ,"image_h")
	call to_history("o2",0.        ,"image_h")
	call to_history("d2",1.        ,"image_h")
	call to_history("esize",8      ,"image_h")
	call to_history("n1",count_mzig,"image_g")
	call to_history("o1",0.        ,"image_g")
	call to_history("d1",1.        ,"image_g")
	call to_history("n2",1         ,"image_g")
	call to_history("o2",0.        ,"image_g")
	call to_history("d2",1.        ,"image_g")
	call to_history("esize",8      ,"image_g")

	! sort the output arrays
	write(0,*)'count_hi=',count_hi
	call sort_two_arrays(count_hi,hi,zih)
	write(0,*)'count_g=',count_g
	call sort_two_arrays(count_g,g,zig)
	write(0,*)'count_mzih=',count_mzih
	call sort_two_arrays(count_mzih,mzih,zimh)
	write(0,*)'count_mzig=',count_mzig
	call sort_two_arrays(count_mzig,mzig,zimg)

	!! output odcig and adcig moveout curves for this gather
	call srite("odcig",cmplx(hi,zih),8*(count_hi))
	g=180*g/pi
	call srite("adcig",cmplx(g,zig),8*(count_g))

	!! output image sections
	call srite("image_h",cmplx(mzih,zimh),8*(count_mzih))
	call srite("image_g",cmplx(mzig,zimg),8*(count_mzig))

	! deallocate work space
	deallocate(zih,zig,zimh,zimg,hi,g,mzih,mzig)

	call exit(0)
end program image_space_moveouts
