!---------------------------------------------------------------------
! DESCRITION:
!   Program to compute ncmp gathers with nh offsets for one
!   DIFFRACTED multiple reflection from a diffractor sitting on top of
!	a dipping water-bottom. No primary, no specularly-reflected 
!	water-bottom multiple and no primary diffraction.
!---------------------------------------------------------------------
!   Input:
!       ocmp: minimum cmp position (m)
!       ncmp: number of cmps to compute
!       dcmp: cmp interval (m)
!       oh: minimum offset (m)
!       nh: number of offsets
!       dh: offset sampling interval (m)
!       nt: number of time samples
!       dt: time sampling interval (s)
!       v: water velocity (m/s)
!       dip: reflector dip (degrees)
!		xdiff: horizontal location of diffractor (m)
!       fpeak: Ricker wavelet peak frequency
!		verb: =1 for verbose output (debug only)
!
!   EXAMPLES:
!       make_cmps_wbmul_dip.x ocmp=500 ncmp=50 dcmp=10 oh=0 nh=200 dh=20\
!			nt=1000 dt=0.004 v=1500 dip=15 xdiff=3000 fpeak=20 \
!			verb=0 > cmps_wb_dip.H
!       make_cmps_wbmul_dip.x ocmp=2000 ncmp=50 dcmp=20 oh=0 nh=100 dh=20\
!			nt=400 dt=0.008 v=1500 dip=15 xdiff=2500 fpeak=20 \
!			verb=0 > cmps_wb_dip.H
!
program make_cmps_diffmul_dip

	use sep
	use auxsu

	implicit none

	! input variables (described above)
	real 	:: ocmp
	integer :: ncmp
	real	:: dcmp
	real	:: oh
	integer	:: nh
	real	:: dh
	integer	:: nt
	real	:: dt
	real	:: v
	real	:: dip
	real	:: xdiff
	real	:: fpeak
	integer :: verb

	! internal variables
	real,dimension(:),pointer :: wavelet
	integer :: i,icmp,ih
	real :: cmp,hh
	real :: xs,xr,zs,z0,xis,zis,zir,xir,xmsdiff,xm1diff,zm1diff,aux
	real :: l1,l2,l3,l4,ddiff,zdiff,betas
	real :: den,tt
	integer :: iw,lw,it0
	real,dimension(5) :: xray,zray
	real,allocatable,dimension(:) :: temp,trace
	
	! intialize SEP stuff
	call initpar()

	! get processing parameters from the command line and set their defaults
	call from_param("ocmp" ,ocmp ,2000.)
	call from_param("ncmp" ,ncmp ,   1 )
	call from_param("dcmp" ,dcmp ,   1.)
	call from_param("oh"   ,oh   ,   0.)
	call from_param("nh"   ,nh   ,   1 )
	call from_param("dh"   ,dh   ,   1.)
	call from_param("nt"   ,nt   , 400 )
	call from_param("dt"   ,dt   ,0.008)
	call from_param("v"    ,v    ,2500.)
	call from_param("dip"  ,dip  ,   0.)
	call from_param("xdiff",xdiff,2500.)
	call from_param("fpeak",fpeak,  20.)
	call from_param("verb" ,verb ,   0 )
write(0,*)'v=',v,'dip=',dip,'xdiff=',xdiff,'fpeak=',fpeak

	!! write to history file
	call to_history("n1",nt)
	call to_history("n2",nh)
	call to_history("n3",ncmp)
	call to_history("o1",0.)
	call to_history("o2",oh)
	call to_history("o3",ocmp)
	call to_history("d1",dt)
	call to_history("d2",dh)
	call to_history("d3",dcmp)
	call to_history("label1","time")
	call to_history("label2","offset")
	call to_history("label3","cmp")
	call to_history("esize",4)

	!! convert dip angle to radians
	dip=dip*pi/180.
	
	!! generate ricker wavelet
	call rickersu(fpeak,dt,lw,iw,wavelet)
	!write(0,*)'wavelet=',wavelet
	
	!! allocate working arrays
	allocate(trace(nt),temp(nt))

	!! loop over cmps
	do icmp=1,ncmp
	
		! set coordinate of this cmp
		cmp=ocmp+(icmp-1)*dcmp
		write(0,*)'cmp=',cmp

		! loop over offsets
		do ih=1,nh

			! set offset of current trace
			! avoid computing hh=0 because of a 0/0 division
			hh=(oh+(ih-1)*dh)/2.
			if (hh==0) then 
				hh=1.0
			end if

			! compute source and receiver coordinates of this trace
			xs=cmp-hh
			xr=cmp+hh

			! compute vertical distance between the shot and the reflector
			zs=xs*tan(dip)

			! compute perpendicular distance between the shot and the reflector
			z0=zs*cos(dip)
			zdiff=xdiff*tan(dip);

			! compute coordinates of image source and image receiver
			xis=xs-2*z0*sin(dip)
			zis=2*z0*cos(dip)

			! compute the location of the multiple bounce at the surface
			xmsdiff=(xis*zdiff+zis*xdiff)/(zdiff+zis)
		
			! compute coordinates of first bounce of multiple at the reflector
			den=zis*(xs-z0*sin(dip))-z0*cos(dip)*(xis-xmsdiff)
			zm1diff=(z0*xmsdiff*zis*cos(dip))/den
			xm1diff=(zm1diff*(xis-xmsdiff)+zis*xmsdiff)/zis

			! compute takeoff angle with repect to the vertical
			ddiff=sqrt(zis**2+(xmsdiff-xis)**2)
			betas=asin((xmsdiff-xs)*cos(dip)/ddiff)-dip;

			! compute the traveltimes of each leg of the multiple
			aux=zs*cos(dip)
			l1=aux/cos(dip+betas)
			l2=(aux*cos(betas))/(cos(dip+betas)*cos(2*dip+betas))
			l3=(aux*cos(betas))/(cos(2*dip+betas)*cos(3*dip+betas))
			l4=sqrt((xr-xdiff)**2+zdiff**2)
			tt=(l1+l2+l3+l4)/v
				
			! compute the trace
			temp=0.
			it0=nint(tt/dt)
			if (verb==1.and.ih==1) then
				write(0,*)'tt=',tt,'it0=',it0
			write(0,*)'hh=',hh,'tt=',tt,'it0=',it0
			write(0,*)'l1=',l1,'l2=',l2,'l3=',l3,'l4=',l4
			end if

			temp(it0)=1.0
			if (verb==1.and.ih==1) then
				write(0,*)'it0=',it0
			end if
			call convsu(lw,iw+1,wavelet,nt,1,temp,nt,1,trace)
			!write(0,*)'temp=',temp
			!write(0,*)'trace=',trace

			! output this trace
			call srite("out",trace,4*nt)
		end do
	end do

write(0,*)' Before deallocate'
	! deallocate working space
	deallocate(trace,temp)

write(0,*)' After deallocate'
	call exit (0)
end program make_cmps_diffmul_dip
